@@ -25,6 +25,11 @@ class DuplicateSigMismatchError(Exception):
         Exception.__init__(self, "mismatched duplicate signatures in multisig")
 
 
+class WrongAmountType(Exception):
+    def __init(self):
+        Exception.__init__(self, "amount (amt) must be a non-negative integer")
+
+
 class WrongChecksumError(Exception):
     def __init__(self):
         Exception.__init__(self, "checksum failed to validate")
@@ -60,6 +65,16 @@ class WrongLeaseLengthError(Exception):
         Exception.__init__(self, "lease length must be 32 bytes")
 
 
+class WrongNoteType(Exception):
+    def __init(self):
+        Exception.__init__(self, "note must be of type \"bytes\"")
+
+
+class WrongNoteLength(Exception):
+    def __init(self):
+        Exception.__init__(self, "note length must be at most 1024")
+
+
 class InvalidProgram(Exception):
     def __init__(self, message="invalid program for logic sig"):
         Exception.__init__(self, message)
@@ -109,7 +124,9 @@ class KMDHTTPError(Exception):
 
 
 class AlgodHTTPError(Exception):
-    pass
+    def __init__(self, msg, code=None):
+        super().__init__(msg)
+        self.code = code
 
 class IndexerHTTPError(Exception):
     pass
